from __future__ import annotations

from typing import Union, Literal, List

from pydantic import Field

from scale_egp.sdk.enums import QuestionType, EvaluationType
from scale_egp.utils.model_utils import BaseModel, RootModel


class CategoricalChoice(BaseModel):
    """
    A choice for a categorical question.

    This is only used in a StudioEvaluationConfig to specify a choice for a  question that will be
    asked to users when they are evaluating generated outputs in the EGP annotation UI.

    Attributes:
        label: The text displayed to annotators for this choice.
        value: The value reported in the TestCaseResult for this question if this choice is
            selected.

            If users would like to track the improvement of a model over time, it is
            recommended to use a numeric value for this field.

            A string value can be used if no ordering is desired.
    """

    label: str
    value: Union[str, int, bool]


class CategoricalQuestion(BaseModel):
    """
    A categorical question.

    This is only used in a StudioEvaluationConfig to specify a question that will be asked to
    users when they are evaluating generated outputs in the EGP annotation UI.

    Attributes:
        question_id: A unique id for this question. Each question_id will be represented as a
            key in the `result` field in the TestCaseResult after an annotation is completed.
            When examining how an application has performed on a specific question, users should
            look at the `result` field in the TestCaseResult for the key with the
            corresponding id.
        question_type: The type of the question.
        title: The text displayed to annotators for this question.
        choices: The choices for the question.
    """

    question_id: str
    question_type: Literal[QuestionType.CATEGORICAL] = QuestionType.CATEGORICAL.value
    title: str
    choices: List[CategoricalChoice]


class DropdownQuestion(BaseModel):
    """
    A dropdown question.

    This is only used in a StudioEvaluationConfig to specify a question that will be asked to
    users when they are evaluating generated outputs in the EGP annotation UI.

    Attributes:
        question_id: A unique id for this question. Each question_id will be represented as a
            key in the `result` field in the TestCaseResult after an annotation is completed.
            When examining how an application has performed on a specific question, users should
            look at the `result` field in the TestCaseResult for the key with the
            corresponding id.
        question_type: The type of the question.
        title: The text displayed to annotators for this question.
        choices: The choices for the question.
    """

    question_id: str
    question_type: Literal[QuestionType.DROPDOWN] = QuestionType.DROPDOWN.value
    title: str
    choices: List[CategoricalChoice]


class FreeTextQuestion(BaseModel):
    """
    A free text question.

    This is only used in a StudioEvaluationConfig to specify a question that will be asked to
    users when they are evaluating generated outputs in the EGP annotation UI.

    Attributes:
        question_id: A unique id for this question. Each question_id will be represented as a
            key in the `result` field in the TestCaseResult after an annotation is completed.
            When examining how an application has performed on a specific question, users should
            look at the `result` field in the TestCaseResult for the key with the
            corresponding id.
        question_type: The type of the question.
        title: The text displayed to annotators for this question.
    """

    question_id: str
    question_type: Literal[QuestionType.FREE_TEXT] = QuestionType.FREE_TEXT.value
    title: str


class Question(RootModel):
    __root__: Union[CategoricalQuestion, DropdownQuestion, FreeTextQuestion] = Field(
        ...,
        discriminator="question_type",
    )


class StudioEvaluationConfig(BaseModel):
    """
    This specifies the configuration for a studio evaluation job.

    Users should use this evaluation config if they intend to do human evaluation through
    [Studio](https://scale.com/studio).

    Attributes:
        evaluation_type: The type of the evaluation.
        studio_project_id: The ID of the Studio project to use for the evaluation.
        questions: The questions to ask users when they are evaluating generated outputs in the
            EGP annotation UI.
    """

    evaluation_type: EvaluationType = EvaluationType.STUDIO
    studio_project_id: str
    questions: List[Question]


EvaluationConfig = StudioEvaluationConfig



from typing import Literal, Optional, Union

from pydantic import BaseModel, Field

from scale_egp.utils.model_utils import RootModel


class ToolRequest(BaseModel):
    """
    Describes a request to run a tool

    Attributes:
        name: Name of the tool
        arguments: Arguments (JSON serializable string) to pass to the tool
    """
    name: str  # Name of the tool
    arguments: str  # Arguments (JSON serializable string) to pass to the tool


class AgentMessage(BaseModel):
    """
    Message from an agent

    Attributes:
        role: The role of the message, must be "agent"
        content: Output of the agent if finished
        tool_request: Request to run a tool
    """
    role: Literal["agent"] = "agent"
    content: Optional[str] = None  # Output of the agent if finished
    tool_request: Optional[ToolRequest] = None  # Request to run a tool


class ToolMessage(BaseModel):
    """
    Message from a tool

    Attributes:
        role: The role of the message, must be "tool"
        name: Name of the tool
        content: Output of calling the tool (JSON serialized to string)
    """
    role: Literal["tool"] = "tool"
    name: str  # Name of the tool
    content: str  # Output of calling the tool (JSON serialized to string)


class UserMessage(BaseModel):
    """
    Message from the user

    Attributes:
        role: The role of the message, must be "user"
        content: The content of the message
    """
    role: Literal["user"] = "user"
    content: str


class AssistantMessage(BaseModel):
    """
    Message from a non-agent AI.

    Attributes:
        role: The role of the message, must be "assistant"
        content: The content of the message
    """
    role: Literal["assistant"] = "assistant"
    content: str


class SystemMessage(BaseModel):
    """
    Message from the system. This is used if this message did not originate from the user or AI.

    Attributes:
        role: The role of the message, must be "system"
        content: The content of the message
    """
    role: Literal["system"] = "system"
    content: str


class Message(RootModel):
    """
    A type alias for a Union of all message types.

    Attributes:
        __root__: Instead of directly using this class, please use the appropriate message type
            for your use case.
    """
    __root__: Union[UserMessage, ToolMessage, AgentMessage, AssistantMessage, SystemMessage] = (
        Field(..., discriminator="role")
    )




from __future__ import annotations

from datetime import datetime
from typing import Optional, List, Dict, Literal

from pydantic import Field

from scale_egp.sdk.enums import ModelEndpointType, ModelType, ModelVendor, GPUType
from scale_egp.sdk.types.models import ParameterSchema
from scale_egp.utils.model_utils import Entity, BaseModel


class ModelBundleConfig(BaseModel):
    """
    Configuration that describes where to download the model execution code from.

    Attributes:
        registry: The docker registry to pull the image from.
        image: The docker image to pull.
        tag: The tag of the docker image to pull.
        command: The command to run when starting the model.
        env: The environment variables to set when starting the model.
        readiness_initial_delay_seconds: The number of seconds to wait before checking if the model
            is ready.
    """
    registry: str
    image: str
    tag: str
    command: List[str] = Field(default_factory=list)
    env: Dict[str, str] = Field(default_factory=dict)
    readiness_initial_delay_seconds: int = Field(120)

    @property
    def full_repository_name(self):
        return "/".join([self.registry, self.image])


class ModelEndpointConfig(BaseModel):
    """
    Configuration for a model endpoint.

    Attributes:
        cpus: The number of CPUs to allocate to each worker.
        memory: The amount of memory to allocate to each worker.
        storage: The amount of storage to allocate to each worker.
        gpus: The number of GPUs to allocate to each worker.
        min_workers: The minimum number of workers to keep running.
        max_workers: The maximum number of workers to keep running.
        per_worker: The number of requests to process per worker before shutting down.
        gpu_type: The type of GPU to allocate to each worker.
        endpoint_type: The type of endpoint to launch.
        high_priority: Whether or not to launch the endpoint with high priority.
    """
    cpus: int = Field(3)
    memory: str = Field("8Gi")
    storage: str = Field("16Gi")
    gpus: int = Field(0)
    # By default, we create model endpoints with min_workers = 0 so unused model endpoints can be
    # autoscaled down to
    # 0 workers, costing nothing.
    min_workers: int = Field(0)
    max_workers: int = Field(1)
    per_worker: int = Field(10)
    gpu_type: Optional[GPUType] = Field(None)
    endpoint_type: ModelEndpointType = Field(ModelEndpointType.ASYNC)
    high_priority: Optional[bool] = Field(False)


class LaunchVendorConfiguration(BaseModel):
    """
    Configuration for launching a model using the Launch service which is an internal and
    self-hosted service developed by Scale that deploys models on Kubernetes.

    Attributes:
        vendor: The vendor of the model template
        bundle_config: The bundle configuration of the model template
        endpoint_config: The endpoint configuration of the model template
    """
    # this field is required for forward compatibility (other providers will have different
    # "vendor" fields)
    vendor: Literal[ModelVendor.LAUNCH] = Field(ModelVendor.LAUNCH)
    bundle_config: ModelBundleConfig
    endpoint_config: Optional[ModelEndpointConfig]


# TODO: This is a copy of LaunchVendorConfiguration. Make this a discriminated union when we have
#  more than one vendor.
class ModelVendorConfiguration(LaunchVendorConfiguration):
    """
    Configuration for launching a model using the Launch service which is an internal and
    self-hosted service developed by Scale that deploys models on Kubernetes.

    Attributes:
        vendor: The vendor of the model template
        bundle_config: The bundle configuration of the model template
        endpoint_config: The endpoint configuration of the model template
    """


class ModelTemplate(Entity):
    """
    This is a template for types of models that can be quickly customized by end users.
    It allows users to upload static docker images that can run specific types of models.
    These docker images will expose parameters that can be injected at ModelAlias creation
    time to customize the functionality. A common example of this is to use a
    HuggingFace LLM template, but swap out model weights for a finetuned model.

    Attributes:
        id: The unique identifier of the entity.
        created_at: The date and time when the entity was created in ISO format.
        account_id: The ID of the account that owns the given entity.
        created_by_user_id: The user who originally created the entity.
        name: The name of the model template
        endpoint_type: The type of endpoint that the model template will create
        model_type: The type of model that the model template will create
        vendor_configuration: The vendor configuration of the model template
        model_creation_parameters_schema: The field names and types of available parameter fields
            which may be specified during model creation
        model_request_parameters_schema: The field names and types of available parameter fields
            which may be specified in a model execution API's `model_request_parameters` field.
    """

    name: str
    endpoint_type: ModelEndpointType
    model_type: ModelType
    vendor_configuration: ModelVendorConfiguration
    model_creation_parameters_schema: Optional[ParameterSchema] = Field(
        None,
        description="The field names and types of available parameter fields which may be "
                    "specified during model creation",
    )
    model_request_parameters_schema: Optional[ParameterSchema] = Field(
        None,
        description="The field names and types of available parameter fields which may be "
                    "specified in a model execution API's `model_request_parameters` field.",
    )
    id: str = Field(..., description="The unique identifier of the entity.")
    created_at: datetime = Field(
        ..., description="The date and time when the entity was created in ISO format."
    )
    account_id: str = Field(..., description="The ID of the account that owns the given entity.")
    created_by_user_id: str = Field(..., description="The user who originally created the entity.")


class ModelTemplateRequest(BaseModel):
    name: str
    endpoint_type: ModelEndpointType
    model_type: ModelType
    vendor_configuration: ModelVendorConfiguration
    model_creation_parameters_schema: Optional[ParameterSchema] = Field(
        None,
        description="The field names and types of available parameter fields which may be "
                    "specified during model creation",
    )
    model_request_parameters_schema: Optional[ParameterSchema] = Field(
        None,
        description="The field names and types of available parameter fields which may be "
                    "specified in a model execution API's `model_request_parameters` field.",
    )
    account_id: str = Field(..., description="The ID of the account that owns the given entity.")



from typing import Tuple, Type, Dict

from pydantic import BaseModel

from scale_egp.sdk.types.models import (
    AgentResponse, AgentRequest, ChatCompletionRequest,
    ChatCompletionResponse, CompletionRequest, CompletionResponse, EmbeddingRequest,
    EmbeddingResponse, RerankingRequest, RerankingResponse,
)
from scale_egp.sdk.enums import ModelType

MODEL_SCHEMAS: Dict[ModelType, Tuple[Type[BaseModel], Type[BaseModel]]] = {
    ModelType.COMPLETION: (CompletionRequest, CompletionResponse),
    ModelType.CHAT_COMPLETION: (ChatCompletionRequest, ChatCompletionResponse),
    ModelType.AGENT: (AgentRequest, AgentResponse),
    ModelType.RERANKING: (RerankingRequest, RerankingResponse),
    ModelType.EMBEDDING: (EmbeddingRequest, EmbeddingResponse),
}



from __future__ import annotations

from datetime import datetime
from typing import Optional, Dict, Any
from warnings import warn

from pydantic import ValidationError, Field, validator

from scale_egp.sdk.enums import EvaluationStatus, TestCaseSchemaType
from scale_egp.sdk.types.evaluation_dataset_test_cases import ExtraInfo
from scale_egp.utils.model_utils import Entity, BaseModel


class GenerationTestCaseResultData(BaseModel):
    """
    A data model representing the data of a TestcaseResult with a GENERATION schema type.

    Attributes:
        generation_output: The output of the generation model
        generation_extra_info: The extra info of the generation model
    """

    generation_output: str
    generation_extra_info: Optional[ExtraInfo] = None


class TestCaseResult(Entity):
    """
    A data model representing a test case result.

    Attributes:
        id: The ID of the test case result
        status: The status of the test case result
        application_spec_id: The ID of the application spec that the test case result is for
        evaluation_id: The ID of the evaluation that the test case result is for
        evaluation_dataset_id: The ID of the evaluation dataset that the test case result is for
        evaluation_dataset_version_num: The version number of the evaluation dataset that the
            test case result is for
        test_case_id: The ID of the test case that the test case result is for
        test_case_evaluation_data: A payload representing the data generated by the application
            described by the application spec when evaluated against the test case.
        test_case_evaluation_data_schema: The schema type of the `test_case_evaluation_data`
        result: The payload filled in when the evaluation of this test case result is completed.
            Examine this value to determine how the application performed on the test case.
        completed_at: The time the test case result was completed
        created_at: The time the test case result was created
    """

    id: str
    label_status: EvaluationStatus
    application_spec_id: str
    evaluation_id: str
    evaluation_dataset_id: str
    evaluation_dataset_version_num: str
    test_case_id: str
    test_case_evaluation_data: GenerationTestCaseResultData
    test_case_evaluation_data_schema: TestCaseSchemaType
    created_at: datetime
    result: Optional[Dict[str, Any]] = None
    completed_at: Optional[datetime] = None

    @property
    def status(self):
        warn(
            message="TestCaseResult.status is deprecated. "
                    "Please use TestCaseResult.label_status instead.",
            category=DeprecationWarning,
        )
        return self.label_status


class TestCaseResultSchemaValidator:
    TEST_CASE_RESULT_SCHEMA_TO_DATA_TYPE = {
        TestCaseSchemaType.GENERATION: GenerationTestCaseResultData,
    }

    @classmethod
    def validate(cls, schema_type: TestCaseSchemaType, data: Dict[str, Any]):
        try:
            model = cls.TEST_CASE_RESULT_SCHEMA_TO_DATA_TYPE[schema_type]
            model.from_dict(data)
        except ValidationError as e:
            raise ValueError(
                f"Test case result data does not match schema type {schema_type}"
            ) from e

    @classmethod
    def get_model(cls, schema_type: TestCaseSchemaType):
        return cls.TEST_CASE_RESULT_SCHEMA_TO_DATA_TYPE[schema_type]

    @classmethod
    def dict_to_model(cls, schema_type: TestCaseSchemaType, data: Dict[str, Any]):
        return cls.TEST_CASE_RESULT_SCHEMA_TO_DATA_TYPE[schema_type].from_dict(data)


class TestCaseResultRequest(BaseModel):
    application_spec_id: str
    evaluation_dataset_version_num: int
    test_case_id: str
    test_case_evaluation_data_schema: TestCaseSchemaType
    test_case_evaluation_data: GenerationTestCaseResultData
    result: Optional[Dict[str, Any]] = Field(default_factory=dict)
    account_id: Optional[str] = Field(
        description="Account to create knowledge base in. If you have access to more than one "
                    "account, you must specify an account_id"
    )

    @validator("test_case_evaluation_data")
    @classmethod
    def test_case_evaluation_data_matches_schema_type(cls, test_case_evaluation_data, values):
        schema_type = values.get("test_case_evaluation_data_schema")
        TestCaseResultSchemaValidator.validate(schema_type, test_case_evaluation_data)
        return test_case_evaluation_data



import json
from typing import Any, Dict, Optional, Type

from pydantic import BaseModel, create_model
from scale_egp.cli.formatter import Markdownable
from scale_egp.sdk.enums import ModelVendor, ModelType
from scale_egp.sdk.types.model_templates import ModelTemplate
from scale_egp.sdk.types.models import (
    Model, ParameterSchema, ParameterSchemaModelConfig,
    parameter_schema_to_model, BaseModelRequest,
)
from scale_egp.sdk.constants.model_schemas import MODEL_SCHEMAS


class ModelDescription(BaseModel, Markdownable):
    model_instance: Model
    model_template: ModelTemplate

    def _get_request_schema(self) -> Dict[str, Any]:
        return get_full_request_schema(
            self.model_template.model_type,
            self.model_template.model_request_parameters_schema,
        ).schema()

    def _get_description_dict(self) -> Dict[str, Any]:
        return {
            "model_instance": json.loads(self.model_instance.json()),
            "model_template": json.loads(self.model_template.json()),
        }

    def to_markdown(self) -> str:
        return (
            f"# {self.model_instance.name} (id: {self.model_instance.id})\n"
            f"\n"
            f"*type*: {self.model_template.model_type.value}\n"
            f"*status*: {self.model_instance.status}\n"
            f"*vendor*: {(self.model_instance.model_vendor or ModelVendor.LAUNCH).value}\n"
            f"\n"
            f"{self.model_instance.description or ''}\n"
            f"## Model request schema\n"
            f"```\n"
            f"{json.dumps(self.model_instance.request_schema, indent=2)}\n"
            f"```\n"
            f"## Model response schema\n"
            f"```\n"
            f"{json.dumps(self.model_instance.response_schema, indent=2)}\n"
            f"```\n"
        )

    def json(self, **dumps_kwargs: Any) -> str:
        return json.dumps(self._get_description_dict(), **dumps_kwargs)


def get_full_request_schema(
    model_type: ModelType,
    model_request_parameter_schema: Optional[ParameterSchema] = None,
) -> Type[BaseModelRequest]:
    # dynamically construct Pydantic model which fully describes the model request schema
    # including the model_request_parameters field if it is used by the model.
    request_schema = create_model(
        MODEL_SCHEMAS[model_type][0].__name__,
        __config__=ParameterSchemaModelConfig,
    )
    # copy over all fields except model_request_parameters
    fields = {
        field_name: field_info
        for field_name, field_info in MODEL_SCHEMAS[model_type][0].__fields__.items()
        if field_name != "model_request_parameters"
    }
    if model_request_parameter_schema and len(model_request_parameter_schema.parameters) > 0:
        request_parameter_schema_cls = parameter_schema_to_model(
            "model_request_parameter_schema",
            model_request_parameter_schema,
        )
        fields["model_request_parameters"] = (request_parameter_schema_cls, ...)
    else:
        # permit an empty model_request_parameters to be sent since pydantic will send it automatically
        fields["model_request_parameters"] = BaseModelRequest.__fields__["model_request_parameters"]

    request_schema.__fields__ = fields
    return request_schema



from __future__ import annotations

from enum import Enum


class TestCaseSchemaType(str, Enum):
    """
    An enum representing the different test case schema types.

    Attributes:
        GENERATION: If a TestCase specifies this schema type, it must have the following fields:

            - `input` [Required] The input to the generation model.
            - `expected_output` [Optional] The expected output of the generation model.
            - `expected_extra_info` [Optional] The expected extra info of the generation model.

            If a TestCaseResult specifies this schema type, it must have the following fields:

            - `generation_output` [Required] The output of the generation model.
            - `generation_extra_info` [Required] The extra info of the generation model.
    """

    GENERATION: str = "GENERATION"


class EvaluationStatus(str, Enum):
    """
    An enum representing the different possible statuses of an Evaluation.

    Attributes:
        PENDING: Denotes that an evaluation is pending.
        COMPLETED: Denotes that an evaluation is completed.
        FAILED: Denotes that an evaluation has failed.
    """

    PENDING: str = "PENDING"
    COMPLETED: str = "COMPLETED"
    FAILED: str = "FAILED"


class AuditStatus(str, Enum):
    """
    An enum representing the different possible audit statuses of a test case result.

    Attributes:
        UNAUDITED: Denotes that the test case result has not been aduited yet.
        FIXED: Denotes the test case result has been audited, and the mistakes have been corrected.
        APPROVED: Denotes the test case has been audited and approved.
    """
    UNAUDITED = "UNAUDITED"
    FIXED = "FIXED"
    APPROVED = "APPROVED"


class EvaluationType(str, Enum):
    """
    An enum representing the different types of evaluations.

    Currently only studio evaluations are supported.

    Attributes:
        STUDIO: Denotes that an evaluation is a studio evaluation.
    """

    STUDIO: str = "studio"


class QuestionType(str, Enum):
    """
    An enum representing the different types of questions.

    This is only used in a StudioEvaluationConfig to specify the type of a question.

    Attributes:
        CATEGORICAL: Denotes that a question is a categorical question.
        DROPDOWN: Denotes that a question is a dropdown question.
        FREE_TEXT: Denotes that a question is a free text question.
    """

    CATEGORICAL: str = "categorical"
    DROPDOWN: str = "dropdown"
    FREE_TEXT: str = "free_text"


class ExtraInfoSchema(str, Enum):
    """
    An enum representing the different types of extra info schemas.

    Denotes the type of the "info" field in the ExtraInfo model.

    Attributes:
        STRING: Denotes that the "info" field is a string.
    """

    STRING: str = "STRING"


class EmbeddingModelName(str, Enum):
    """
    An enum representing the different types of embedding models supported.

    Attributes:
        SENTENCE_TRANSFORMERS_ALL_MINILM_L12_V2:
            Denotes that the model is a sentence transformer model.
        SENTENCE_TRANSFORMERS_MULTI_QA_DISTILBERT_COS_V1:
            Denotes that the model is a sentence transformer model.
        SENTENCE_TRANSFORMERS_PARAPHRASE_MULTILINGUAL_MPBET_BASE_V2:
            Denotes that the model is a sentence transformer model.
        OPENAI_TEXT_EMBEDDING_ADA_002:
            Denotes that the model is an openai text embedding model.
    """

    SENTENCE_TRANSFORMERS_ALL_MINILM_L12_V2 = "sentence-transformers/all-MiniLM-L12-v2"
    SENTENCE_TRANSFORMERS_MULTI_QA_DISTILBERT_COS_V1 = (
        "sentence-transformers/multi-qa-distilbert-cos-v1"
    )
    SENTENCE_TRANSFORMERS_PARAPHRASE_MULTILINGUAL_MPBET_BASE_V2 = (
        "sentence-transformers/paraphrase-multilingual-mpnet-base-v2"
    )
    OPENAI_TEXT_EMBEDDING_ADA_002 = "openai/text-embedding-ada-002"
    SENTENCE_TRANSFORMERS_MULTI_QA_FINETUNE = "finetuned/multi-qa-mpnet-base-dot-v1"


class DataSource(str, Enum):
    """
    An enum representing the different types of data sources supported.

    Attributes:
        S3: Denotes that the data source is S3.
    """

    S3: str = "S3"
    SHAREPOINT: str = "SharePoint"
    GOOGLE_DRIVE: str = "GoogleDrive"
    LOCAL_CHUNKS: str = "LocalChunks"


class DeduplicationStrategy(str, Enum):
    """
    An enum representing the different types of deduplication strategies supported.

    Attributes:
        OVERWRITE: Denotes that the deduplication strategy is to overwrite.
        FAIL: Denotes that the deduplication strategy is to fail.
    """
    OVERWRITE = "Overwrite"
    FAIL = "Fail"


class ChunkingStrategy(str, Enum):
    """
    An enum representing the different types of chunking strategies supported.

    Attributes:
        CHARACTER: Denotes that the chunking strategy is to chunk by character.
    """
    CHARACTER = "character"


class ArtifactSource(str, Enum):
    """
    An enum representing the different types of artifact sources supported.

    Attributes:
        S3: Denotes that the artifact source is S3.
        CONFLUENCE: Denotes that the artifact source is Confluence.
        SHAREPOINT: Denotes that the artifact source is SharePoint.
        GOOGLE_DRIVE: Denotes that the artifact source is Google Drive.
        LOCAL_CHUNKS: Denotes that the artifact source comes from chunks directly uploaded via the
            API.
    """
    S3 = "S3"
    CONFLUENCE = "Confluence"
    SHAREPOINT = "SharePoint"
    GOOGLE_DRIVE = "GoogleDrive"
    LOCAL_CHUNKS = "LocalChunks"


class StatusType(str, Enum):
    PENDING = "Pending"
    RUNNING = "Running"
    COMPLETED = "Completed"
    FAILED = "Failed"
    CANCELED = "Canceled"
    CHUNKING = "Chunking"
    DELETING = "Deleting"
    UPLOADING = "Uploading"


class UploadJobStatus(str, Enum):
    """
    An enum representing the different types of upload job statuses supported.

    Attributes:
        RUNNING: Denotes that the upload job is running.
        COMPLETED: Denotes that the upload job is completed.
        FAILED: Denotes that the upload job has failed.
        CANCELED: Denotes that the upload job has been canceled.
    """
    RUNNING = StatusType.RUNNING.value
    COMPLETED = StatusType.COMPLETED.value
    FAILED = StatusType.FAILED.value
    CANCELED = StatusType.CANCELED.value


class ChunkUploadStatus(str, Enum):
    """
    An enum representing the different types of chunk upload statuses supported.

    Attributes:
        PENDING: Denotes that the chunk upload is pending.
        COMPLETED: Denotes that the chunk upload is completed.
        FAILED: Denotes that the chunk upload has failed.
    """
    PENDING = StatusType.PENDING.value
    COMPLETED = StatusType.COMPLETED.value
    FAILED = StatusType.FAILED.value


class CrossEncoderModelName(str, Enum):
    """
    An enum representing the different types of cross encoder models supported.

    Attributes:
        CROSS_ENCODER_MS_MARCO_MINILM_L12_V2: Denotes that the model is a cross encoder model.
        CROSS_ENCODER_MMARCO_MMINILMV2_L12_H384_V1: Denotes that the model is a cross encoder model.
    """
    CROSS_ENCODER_MS_MARCO_MINILM_L12_V2 = "cross-encoder/ms-marco-MiniLM-L-12-v2"
    CROSS_ENCODER_MMARCO_MMINILMV2_L12_H384_V1 = "cross-encoder/mmarco-mMiniLMv2-L12-H384-v1"


class ModelState(str, Enum):
    """
    An enum representing the different types of model states supported.

    Attributes:
        ENABLED: Denotes that the model is enabled.
        PENDING: Denotes that the model is pending.
        DISABLED: Denotes that the model is disabled.
    """
    ENABLED = "ENABLED"
    PENDING = "PENDING"
    DISABLED = "DISABLED"


class ModelVendor(str, Enum):
    """
    An enum representing the different types of model vendors supported.

    Attributes:
        OPENAI: Denotes that the model vendor is OpenAI.
        COHERE: Denotes that the model vendor is Cohere.
        ANTHROPIC: Denotes that the model vendor is Anthropic.
        LLMENGINE: Denotes that the model vendor is LLM Engine.
        OTHER: Denotes that the model vendor is Other.
    """
    OPENAI = "OPENAI"
    COHERE = "COHERE"
    ANTHROPIC = "ANTHROPIC"
    LAUNCH = "LAUNCH"
    OTHER = "OTHER"


class ModelEndpointType(str, Enum):
    """
    An enum representing the different types of model endpoint types supported.

    Attributes:
        SYNC: Denotes that the model endpoint type is sync.
        ASYNC: Denotes that the model endpoint type is async.
        STREAMING: Denotes that the model endpoint type is streaming.
        BATCH: Denotes that the model endpoint type is batch.
    """
    SYNC = "SYNC"
    ASYNC = "ASYNC"
    STREAMING = "STREAMING"
    BATCH = "BATCH"


class ModelType(str, Enum):
    """
    An enum representing the different types of models supported.

    Attributes:
        COMPLETION: Denotes that the model type is completion.
        CHAT_COMPLETION: Denotes that the model type is chat completion.
        AGENT: Denotes that the model type is agent.
        EMBEDDING: Denotes that the model type is embedding.
        RERANKING: Denotes that the model type is reranking.
    """
    COMPLETION = "COMPLETION"
    CHAT_COMPLETION = "CHAT_COMPLETION"
    AGENT = "AGENT"
    EMBEDDING = "EMBEDDING"
    RERANKING = "RERANKING"


class AgentAction(str, Enum):
    """
    An enum representing the different types of agent actions supported.

    Attributes:
        TOOL_REQUEST: Denotes that the agent output contains a request for the user to use a tool.
        CONTENT: Denotes that the agent output contains final content.
    """
    TOOL_REQUEST = "tool_request"
    CONTENT = "content"


class GPUType(str, Enum):
    # Supported GPU models according to
    # https://github.com/scaleapi/launch-python-client/blob
    # /794089b9ed58330a7ecac02eb5060bcc4ae3d409/launch/client.py#L1470-L1471
    NVIDIA_TESLA_T4 = "nvidia-tesla-t4"
    NVIDIA_AMPERE_A10 = "nvidia-ampere-a10"
    NVIDIA_AMPERE_A100 = "nvidia-ampere-a100"
    NVIDIA_AMPERE_A100e = "nvidia-ampere-a100e"



from __future__ import annotations

from typing import Literal, Optional, List

from pydantic import Field

from scale_egp.utils.model_utils import BaseModel


class CompletionContent(BaseModel):
    """
    A data model representing the completion text and the finish reason.

    Attributes:
        text: Completion text. If streaming, this field will contain each packet of text.
        finish_reason: Reason the LLM finished generating text.
    """

    text: str
    finish_reason: Optional[str] = None


class TokenUsage(BaseModel):
    """
    A data model representing LLM token usage numbers.

    Attributes:
        prompt: Number of tokens in the prompt.
        completion: Number of tokens in the completion.
        total: Total number of tokens in both the prompt and the completion.
    """

    prompt: Optional[int] = None
    completion: Optional[int] = None
    total: int


class Completion(BaseModel):
    """
    A data model representing a completion.

    Attributes:
        completion: The actual completion text and the finish reason.
        token_usage: Token usage numbers. If streaming, this field is null until the stream
            completes, at which point it will be populated (if supported).
    """

    completion: CompletionContent
    token_usage: Optional[TokenUsage] = None


class ModelParameters(BaseModel):
    """
    A data model representing the configuration parameters for the completion model.

    Attributes:
        temperature: What sampling temperature to use, between [0, 1]. Higher values like 0.8
            will make the output more random, while lower values like 0.2 will make it more
            focused and deterministic. Setting temperature=0.0 will enable fully deterministic
            (greedy) sampling.
        stop_sequences: List of up to 4 sequences where the API will stop generating further
            tokens. The returned text will not contain the stop sequence.
        max_tokens: The maximum number of tokens to generate in the completion. The token count
            of your prompt plus max_tokens cannot exceed the model's context length. If not,
            specified, max_tokens will be determined based on the model used:

            | Model API family | Model API default | EGP applied default |
            | --- | --- | --- |
            | OpenAI Completions | [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens) | `context window - prompt size` |
            | OpenAI Chat Completions | [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens) | `context window - prompt size` |
    """

    temperature: float = Field(default=0.2, ge=0.0, le=1.0)
    stop_sequences: Optional[List[str]] = Field(default=None, max_items=4)
    max_tokens: Optional[int] = Field(default=None)


class CompletionRequest(BaseModel):
    model: Literal[
        "gpt-4",
        "gpt-4-0613",
        "gpt-4-32k",
        "gpt-4-32k-0613",
        "gpt-3.5-turbo",
        "gpt-3.5-turbo-0613",
        "gpt-3.5-turbo-16k",
        "gpt-3.5-turbo-16k-0613",
        "text-davinci-003",
        "text-davinci-002",
        "text-curie-001",
        "text-babbage-001",
        "text-ada-001",
    ] = Field(
        ...,
        description="The ID of the model to use for completions.\n\n"
                    "Users have two options:\n"
                    "- Option 1: Use one of the supported models from the dropdown.\n"
                    "- Option 2: Enter the ID of a custom model.\n\n"
                    "Note: For custom models we currently only support models finetuned using "
                    "using the Scale-hosted LLM-Engine API.",
    )
    prompt: str = Field(
        ...,
        description="Prompt for which to generate the completion.\n\n"
                    "Good prompt engineering is crucial to getting performant results from the "
                    "model. If you are having trouble getting the model to perform well, "
                    "try writing a more specific prompt here before trying more expensive "
                    "techniques such as swapping in other models or finetuning the underlying LLM.",
    )
    model_parameters: Optional[ModelParameters] = Field(
        default=ModelParameters(temperature=0.2),
        description="Configuration parameters for the completion model, such as temperature, "
                    "max_tokens, and stop_sequences.\n\n"
                    "If not specified, the default value are:\n"
                    "- temperature: 0.2\n"
                    "- max_tokens: None (limited by the model's max tokens)\n"
                    "- stop_sequences: None",
    )
    stream: bool = Field(
        default=False,
        description="Whether or not to stream the response.\n\n"
                    "Setting this to True will stream the completion in real-time.",
    )



from __future__ import annotations

from typing import Optional, List, Dict, Any, Union, Literal

from pydantic import Field

from scale_egp.sdk.enums import CrossEncoderModelName
from scale_egp.utils.model_utils import Entity, BaseModel


class Chunk(Entity):
    """
    A data model representing a chunk.

    Attributes:
        chunk_id: The unique ID of the chunk
        text: The text associated with the chunk
        score: A number between 0 and 1 representing how similar a chunk's embedding is to the
            query embedding. Higher numbers mean that this chunk is more similar.
        embedding: The vector embedding of the text associated with the chunk
        metadata: Any additional key value pairs of information stored with the chunk
    """

    chunk_id: str
    text: str
    score: Optional[float]
    embedding: Optional[List[float]]
    metadata: Optional[Dict[str, Any]] = Field(default_factory=dict)


class CrossEncoderRankParams(BaseModel):
    """
    A data model representing the parameters needed for cross encoder ranking.

    Attributes:
        cross_encoder_model: The cross encoder model to use for ranking.
    """
    cross_encoder_model: Literal[
        "cross-encoder/ms-marco-MiniLM-L-12-v2",
        "cross-encoder/mmarco-mMiniLMv2-L12-H384-v1",
    ] = Field(
        default=CrossEncoderModelName.CROSS_ENCODER_MS_MARCO_MINILM_L12_V2,
        description="Cross encoder model to use when ranking.",
    )


class CrossEncoderRankStrategy(BaseModel):
    """
    A data model representing the cross encoder ranking strategy.

    Attributes:
        method: The name of the rank strategy. Must be `cross_encoder`.
        params: The parameters needed for ranking.
    """
    method: Literal["cross_encoder"] = Field(
        default="cross_encoder",
        const=True,
        description="The name of the rank strategy. Must be `cross_encoder`.",
    )
    params: CrossEncoderRankParams = Field(
        default=CrossEncoderRankParams(),
        description="The parameters needed for ranking.",
    )


class RougeRankParams(BaseModel):
    """
    A data model representing the parameters needed for Rouge ranking.

    Attributes:
        metric: Rouge type, can be n-gram based (e.g. rouge1, rouge2) or longest common
            subsequence (rougeL or rougeLsum)
        score: Metric to use from Rouge score
    """
    metric: str = Field(
        default="rouge2",
        description="Rouge type, can be n-gram based (e.g. rouge1, rouge2) or longest common "
        "subsequence (rougeL or rougeLsum)",
    )
    score: Literal["precision", "recall", "fmeasure"] = Field(
        default="recall", description="Metric to use from Rouge score"
    )


class RougeRankStrategy(BaseModel):
    """
    A data model representing the Rouge ranking strategy.

    Attributes:
        method: The name of the rank strategy. Must be `rouge`.
        params: The parameters needed for ranking.
    """
    method: Literal["rouge"] = Field(
        default="rouge",
        const=True,
        description="The name of the rank strategy.",
    )
    params: RougeRankParams = Field(
        default=RougeRankParams(),
        description="The parameters needed for ranking.",
    )


class ModelRankParams(BaseModel):
    """
    A data model representing the parameters needed for ranking.

    Attributes:
        model_id: The ID of the model to use for ranking.
        model_params: The parameters needed for the model.
    """
    model_id: str
    model_params: Optional[Dict[str, Any]] = Field(default_factory=dict)


class ModelRankStrategy(BaseModel):
    """
    A data model representing the model ranking strategy.

    Attributes:
        method: The name of the rank strategy. Must be `model`.
        params: The parameters needed for ranking.
    """
    method: Literal["model"] = Field(
        default="model",
        const=True,
        description="Use a model from Models API for ranking.",
    )
    params: ModelRankParams = Field(
        ...,
        description="The parameters needed for ranking.",
    )


class ChunkRankRequest(BaseModel):
    query: str = Field(
        ...,
        description="Natural language query to re-rank chunks against. If a vector store query "
                    "was originally used to retrieve these chunks, please use the same query for "
                    "this ranking",
    )
    relevant_chunks: List[Chunk] = Field(..., description="List of chunks to rank")
    rank_strategy: Union[CrossEncoderRankStrategy, RougeRankStrategy, ModelRankStrategy] = Field(
        ...,
        discriminator="method",
        description="The ranking strategy to use.\n\n"
                    "Rank strategies determine how the ranking is done, They consist of the "
                    "ranking method name and additional params needed to compute the ranking.",
    )
    top_k: Optional[int] = Field(
        gt=0,
        description="Number of chunks to return. Must be greater than 0 if specified. If not "
                    "specified, all chunks will be returned.",
    )


class ChunkRankResponse(BaseModel):
    relevant_chunks: List[Chunk] = Field(
        ..., description="List of chunks ranked by the requested rank strategy"
    )


class ChunkSynthesisRequest(BaseModel):
    query: str = Field(
        ...,
        description="Natural language query to resolve using the supplied chunks.",
    )
    chunks: List[Chunk] = Field(
        ..., description="List of chunks to use to synthesize the response."
    )


class ChunkSynthesisResponse(BaseModel):
    response: str = Field(..., description="Natural language response addressing the query.")
    metadata: Optional[Dict[str, Dict]] = Field(
        None, description="Optional metadata present on each chunk."
    )
